# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

try:
    import psyco
    psyco.full()
except ImportError:
    print '(no psyco)'

import os

import cPickle

import shlex
    
import traceback

from twisted.internet import reactor

from pySndo.network.server import ServerGameProtocol, ServerGameFactory, \
    ServerInfoProtocol, ServerInfoFactory

from pySndo.network.site.updater import ServerUpdater

from pySndo.map.loader import loadMap

from commands.getcommands import getCommands

from pySndo.network.packetloaders.battle import UseItem, ThrowFlashbang, ThrowSmoke, ThrowMolotov

import random

class MyGameProtocol(ServerGameProtocol):
    rights = 0
    
    def packetCorrupted(self, realData, e):
        file = open('./bug.txt', 'ab')
        file.writelines(['** corrupted: %r **' % realData])
        traceback.print_exc(file = file)
        ServerGameProtocol.packetCorrupted(self, e, realData)
    
    def acceptLogin(self, name):
        ip = self.transport.getPeer().host
        banned = self.factory.banned['Bans']
        if ip in banned['BannedIPs']:
            self.kick(banned['BannedReasons'][ip], True)
            return False
        return True
    
    def loggedIn(self, name):
        print 'logged in, sending message'
        self.sendMessages(self.factory.welcome, 5, 0, 1.5)
        
    def playerKilled(self, player, dieType):
        player.sendMessage('%s had %s HP left' % (self.name, self.player.health))
    
    def acceptMessage(self, message):
        if message.startswith('!'):
            try:
                splitted = shlex.split(message[1:])
                command = self.factory.commands.findCommand(splitted[0])
            except (IndexError, ValueError), e:
                self.sendStatus('Invalid command: %s' % e)
                return False
            if command:
                try:
                    result = command.handle(self, splitted[1:])
                except Exception, e:
                    file = open('./bug.txt', 'ab')
                    file.writelines(['** corrupted command: %r **' % message])
                    traceback.print_exc(file = file)
                    self.sendStatus('Invalid command: %s' % e)
                    return False
                if not result is None:
                    self.sendStatus(result)
            else:
                self.sendStatus('Invalid command')
            return False
        return True
            
    def kick(self, *arg, **kw):
        ServerGameProtocol.kick(self, *arg, **kw)
        print 'Kicked %s' % self.name

class MyGameFactory(ServerGameFactory):
    protocol = MyGameProtocol
    commands = None
    timeOut = 2000
    passwords = None
    friendlyFire = False
    logFile = None
    connectionLogFile = None
    
    maxPlayers = 50
    maxPing = 700
    map = loadMap('lts_baseinvasion', './../Maps/')
    
    welcome = open('welcome.txt', 'rb').read().splitlines()
    
    manholeFactory = None
    
    def startFactory(self):
        ServerGameFactory.startFactory(self)
        self.logFile = open('logging.txt', 'ab')
        self.connectionLogFile = open('connectionlog.txt', 'wb')
        self.commands = getCommands()
        if os.path.isfile('./banned.dat'):        
            self.banned = cPickle.load(open('./banned.dat', 'rb'))
        else:
            self.banned = {
                'Hardbans' : [],
                'Bans' : {'BannedIPs' : [],
                          'BannedReasons' : {}}
                }
        if os.path.isfile('./passwords.txt'):
            self.passwords = []
            passwordList = open('./passwords.txt', 'rb').read().splitlines()
            for line in passwordList:
                splitted = line.split(',')
                self.passwords.append((splitted.pop(0), int(splitted.pop(0))))
        else:
            self.passwords = []
            
    def searchPlayers(self, name):
        return [connection for connection in self.connections 
                if connection.name.lower().count(name.lower())]
            
    def connectionFlooding(self, ip):
        self.banned['Hardbans'].append(ip)
        self.saveBans()
    
    def saveBans(self):
        cPickle.dump(self.banned, open('./banned.dat', 'wb'))
        
    def savePasswords(self):
        data = [','.join(item) for item in self.passwords]
        open('./passwords.txt', 'wb').write('\n'.join(data))
        
    def buildProtocol(self, addr):
        if addr.host in self.banned['Hardbans']:
            self.log('%s is hardbanned' % addr.host, True)
            return None
        else:
            return ServerGameFactory.buildProtocol(self, addr)
            
    def log(self, message, connection = False):
        if connection:
            logFile = self.connectionLogFile
        else:
            logFile = self.logFile
        logFile.write(message + '\r\n')

class MyInfoProtocol(ServerInfoProtocol):
    def initialSent(self):
        print '(initial sent)'
        
    def mapTransferStarted(self, map):
        print '(map transfer started)'

    def mapProgress(self, map, currentLines, totalLines):
        pass
    
    def mapSent(self, map):
        print '(map transferred successfully!)'
    
    def mapTransferringSent(self):
        print '(sent mapsending)'

class MyInfoFactory(ServerInfoFactory):
    protocol = MyInfoProtocol
    timeOut = 2000

    welcome = None
    
    def __init__(self, *arg, **kw):
        ServerInfoFactory.__init__(self, *arg, **kw)
    
    def buildProtocol(self, addr):
        if addr.host in self.gameFactory.banned['Hardbans']:
            return None
        else:
            return ServerInfoFactory.buildProtocol(self, addr)

def main():
    basePort = 1203
    serverName = 'pySndo example %s' % random.randrange(0, 2000)

    gameFactory = MyGameFactory()
    infoFactory = MyInfoFactory(gameFactory)
    infoPort = reactor.listenTCP(basePort+1, infoFactory)
    gamePort = reactor.listenTCP(basePort, gameFactory)

    newUpdater = ServerUpdater(serverName, infoFactory, basePort)
    print 'Listening on port %s and %s...' % (infoPort.port, gamePort.port)
    reactor.run()
    
if __name__ == '__main__':
    main()
