# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

from twisted.internet import reactor, defer

from twisted.internet.task import LoopingCall

# The default lifetime for a servermessage in Snd is 10 seconds
# To bypass this, we resend the servermessage every 8 second (if it is not empty, that is!)

class Message(object):
    message = None
    lifetime = None
    delay = None

    def __init__(self, message, lifetime):
        self.message = message
        self.lifetime = lifetime

class MessageList(object):
    pendingItems = None
    items = None
    currentItems = None
    defaultLifetime = None
    callbackFunction = None
    _resendInterval = None
    _resender = None
    
    def __init__(self, callbackFunction, defaultLifetime = 5, resendInterval = 8):
        self.defaultLifetime = defaultLifetime
        self.callbackFunction = callbackFunction
        resender = LoopingCall(self._resend)
        resender.start(resendInterval, False)
        self._resender = resender
        self._resendInterval = resendInterval
        self.pendingItems = []
        self.items = []
        
    def getMessages(self):
        return [item.message for item in self.items]
        
    def addMessage(self, message, lifetime = None, delay = None):
        """
        Add a message to the list.
        
        @param message: Message string to add.
        @param lifetime: Lifetime of the message.
        @param delay: Number of seconds to wait before
            adding the message to the list.
        """
        lifetime = lifetime or self.defaultLifetime
        if delay:
            reactor.callLater(delay, self.addMessage, message, lifetime)
        else:
            if message == '':
                lines = ['']
            else:
                lines = message.splitlines()
            for line in lines:
                newMessage = Message(line, lifetime)
                if len(self.items) < 6:
                    self.items.append(newMessage)
                    reactor.callLater(lifetime, self._removeMessage, newMessage)
                    self._listUpdated()
                else:
                    self.pendingItems.append(newMessage)
        
    def addMessages(self, messageList, lifetime = None, delay = 0, delayStep = 0):
        """
        Add a list of messages to the list.
        
        @param delayStep: If not 0, this will be the
            step-up for the delay for each message (useful for
            scrolling messages)
        """
        for message in messageList:
            self.addMessage(message, lifetime, delay)
            delay += delayStep
            
    def _resend(self):
        if self.items:
            self._listUpdated()
            
    def _listUpdated(self):
        self.callbackFunction(self)
        
    def _removeMessage(self, message):
        if len(self.items) >= 6 and self.pendingItems:
            newMessage = self.pendingItems.pop(0)
            self.items.append(newMessage)
            reactor.callLater(newMessage.lifetime, self._removeMessage, newMessage)
        self.items.remove(message)
        self._resender.stop()
        self._resender.start(self._resendInterval, False)
        self._listUpdated()