# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

import binascii

import os

from pySndo.locals import *

# change this to look up in the mapObjects array
from pySndo.map.objects import Object

class MapNotFound(Exception):
    """
    Raised when the selected map was not found
    """
    
class Data(object):
    uniqueID = None
    background = None
    author = None
    gameType = None
    objects = None
    spawns = None
    def __init__(self, dataLines):
        objects = []
        spawns = []
        del dataLines[-2:] # useless lines!
        self.uniqueID = int(dataLines.pop(0))
        background = dataLines.pop(0).split(',')
        if len(background) > 1:
            self.author = background[1]
        self.background = int(background[0])
        self.gameType = int(dataLines.pop(0))
        for line in dataLines:
            newObject = Object(line)
            objects.append(newObject)
        self.objects = objects
        self.spawns = spawns
        
    def getGameType(self):
        return mapGameTypes[self.gameType]
        
    def setGameType(self, name):
        for key, value in mapGameTypes:
            if value == name:
                self.gameType = key
                return True
        return False
        
    def getBackground(self):
        return mapBackgroundTypes[self.background]
        
    def setBackground(self, name):
        for key, value in mapBackgroundTypes:
            if value == name:
                self.background = key
                return True
        return False
        
    def getSpawns(self, spawnType = None):
        if spawnType == 'Terrorist':
            objectType = ('Terrorist Spawn Point')
        elif spawnType == 'Police':
            objectType = ('Police Spawn Point')
        elif spawnType == 'Generic':
            objectType = ('Deathmatch Spawn Point')
        else:
            objectType = ('Terrorist Spawn Point',
                          'Police Spawn Point',
                          'Deathmatch Spawn Point')
        return [object for object in self.objects
                if object.getObjectType() in objectType]

    def generate(self):
        if self.author:
            background = ','.join([str(self.background), self.author])
        else:
            background = self.background
        data = [
            self.uniqueID,
            background,
            self.gameType
        ]
        data.extend([object.generate() for object in self.objects])
        data.extend([len(data)+1, '']) # keeping for compatibility
        return [str(item) for item in data]
    
class Map(object):
    data = None
    name = None
    
    def __init__(self, data = None):
        if data:
            dataLines = shift_decrypt(data, MAP_ENCRYPT_SHIFT).split('\n')
            self.data = Data(dataLines)
            
    def computeChecksum(self):
        return binascii.crc32(self.generate())
            
    def generate(self):
        return shift_encrypt('\n'.join(self.data.generate()), MAP_ENCRYPT_SHIFT)

def loadMap(mapName, *searchPaths):
    """
    Looks in ./Maps/ for a matching map,
    and if one is found, it is returned as and
    L{Map} object
    """
    from pySndo.map.loader import Map
    # search paths
    searchPathList = list(searchPaths)
    searchPathList.extend([os.path.join(os.getcwd(), 'Maps')])
    for path in searchPathList:
        filePath = os.path.join(path, mapName + '.sdo')
        if not os.path.isfile(filePath):
            continue
        else:
            newMap = Map(open(filePath, 'rb').read())
            newMap.name = mapName
            return newMap
    raise MapNotFound('map %r not found in any of the search paths' % mapName)