# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

from pySndo.locals import find_key, create_number

from pySndo.network.packetloaders.welcome import Welcome, SetName
from pySndo.network.packetloaders.playerlist import PlayerList
from pySndo.network.packetloaders.map import CurrentMap, SetMap, MapStart, BombedArea, SpawnItem, GotMedipack
from pySndo.network.packetloaders.chat import Chat, TeamChat, Typing, DoneTyping, ServerMessage
from pySndo.network.packetloaders.ping import Ping, Pong
from pySndo.network.packetloaders.join import JoinGame, RequestSpawn, SpawnGranted
from pySndo.network.packetloaders.team import SetTeam, TeamScore, IncrementScore, SelfKill
from pySndo.network.packetloaders.movement import Moving, Stopped, ChangeDirection, LookDirection, Spawn
from pySndo.network.packetloaders.battle import Reload, Shoot, Hit, HeavyHit, Killed, UseItem, ThrowFlashbang, ThrowSmoke, ThrowMolotov
from pySndo.network.packetloaders.player import PlayerJoined, PlayerLeft, PlayerKicked, PlayerBanned, GotBanned, PlayerKills, SkinChange
from pySndo.network.packetloaders.status import Closing, Restart, ServerFull, TimeLeft, NextMap, ServerStatus

unpaddedTypes = [
    Killed,
    IncrementScore
]

paddedTypes = [
    Hit,
    HeavyHit#,
#    GotMedipack
]

twoTypes = {
    0 : Spawn,
    1 : ChangeDirection,
    2 : Stopped,
    3 : Moving,
    4 : LookDirection,
    5 : DoneTyping,
    6 : Chat,
    7 : SpawnItem,
    8 : Shoot,
    9 : Reload,
    10 : Killed,
    11 : PlayerKills,
    12 : UseItem,
    13 : SetTeam,
    14 : MapStart,
    15 : IncrementScore, # Round End
    16 : ThrowFlashbang,
    17 : SelfKill, # Kill self
    18 : Restart,
    19 : SkinChange,
    20 : Ping,
    21 : TeamChat,
    22 : ServerMessage,
    23 : SetMap,
    24 : ServerStatus,
    25 : Typing,
    40 : ThrowMolotov,
    41 : ThrowSmoke
}

commonOneTypes = {
    #2 : str, # (is On user is here, but not used)
    3 : PlayerJoined,
    4 : PlayerLeft,
    5 : PlayerList,
    6 : ServerFull,
    8 : Closing,
    9 : GotMedipack,
    12 : Hit,
    13 : HeavyHit,
    14 : TeamScore,
    15 : Pong,
    16 : CurrentMap,
    17 : PlayerKicked,
    18 : PlayerBanned,
    19 : GotBanned,
    20 : TimeLeft,
    21 : NextMap,
    22 : BombedArea,
    23 : JoinGame
}

serverOneTypes = commonOneTypes.copy()
serverOneTypes.update({
    1 : Welcome,
    10 : SpawnGranted
})

clientOneTypes = commonOneTypes.copy()
clientOneTypes.update(
{
    1 : SetName, #  SetName
    10 : RequestSpawn
}
)

class Message(object):
    _data = None # remove soon, please
    way = None
    id = None
    type = None
    def __init__(self, data = None, isClient = True):
        self._data = data
        if data:
            self.way = int(data[0])
            id = int(data[1:3])
            self.id = id
            try:
                if self.way == 2:
                    if twoTypes[id] in unpaddedTypes:
                        type = data[3:]
                    else:
                        type = data[3:-1]
                    # TODO: remove
                    try:
                        self.type = twoTypes[id](type)
                    except ValueError, e:
                        print '- error: %r' % data
                        raise e
                elif self.way == 1:
                    type = data[3:]
                    if isClient:
                        oneTypes = serverOneTypes
                    else:
                        oneTypes = clientOneTypes
                    if oneTypes[id] in paddedTypes:
                        type = type[:-1]
                    self.type = oneTypes[id](type)
                else:
                    raise ValueError('%s not handled! (%r)' % (id, data))
            except (IndexError, KeyError):
                raise ValueError('%s not handled! (%r)' % (id, data))
        
    def generate(self):
        data = self.type.generate()
        try:
            id = find_key(twoTypes, self.type.__class__)
            self.way = 2
            if not self.type.__class__ in unpaddedTypes:
                data += '0' # padding for this type
        except ValueError:
            try:
                id = find_key(clientOneTypes, self.type.__class__)
                self.way = 1
            except ValueError:
                try:
                    id = find_key(serverOneTypes, self.type.__class__)
                    self.way = 1
                except ValueError:
                    raise NotImplementedError('could not find matching packet type')
        if self.type.__class__ in paddedTypes:
            data += '0'
        return ''.join([str(item) for item in 
            [
                self.way,
                create_number(id, 2),
                data,
            ]
        ])
