# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

from pySndo.locals import create_number, checkDefault, find_key

from pySndo.network.packetloaders.common import PrefixSend

from pySndo.game.weapon import Glock, SIGP, DesertEagle, Benelli, BenelliHit, \
    MP5, Galil, SVD, AK47, Colt, G3A3, SR60, AW50, GasGrenade, HEGrenade, C4, \
    Flashbang, MolotovCocktail, SmokeGrenade, Medipack, Other

weapons = {
    0 : BenelliHit,
    1 : Glock,
    2 : AK47,
    3 : Benelli,
    4 : DesertEagle,
    5 : AW50,
    6 : MP5,
    7 : G3A3,
    8 : SIGP,
    9 : Colt,
    10 : Galil,
    11 : SVD,
    12 : SR60,
    13 : AK47
}

dieTypes = [
    'Shot',
    'Burned'
]

dieWeapons = {
    GasGrenade : 'Shot',
    HEGrenade : 'Burned',
    C4 : 'Burned',
    MolotovCocktail : 'Burned',
    Other : 'Burned'
}

for weapon in weapons.values():
    dieWeapons[weapon] = 'Shot'

itemList = [
    GasGrenade,
    HEGrenade,
    C4
]

items = {
    1 : GasGrenade,
    2 : Other
}

itemToId = {
    GasGrenade : 1,
    HEGrenade : 2,
    C4 : 2,
    MolotovCocktail : 2,
    Other : 2
}
    

class Reload(PrefixSend):
    reloading = False
    def _load(self, data):
        checkDefault(int(data)-1, 0, 1)
        self.reloading = not bool(int(data)-1)
        
    def _generate(self):
        return create_number(int(not self.reloading)+1, 1)

"""
2805026
1200568
"""

class UseItem(PrefixSend):
    power = None
    direction = None
    weapon = None
    x = None
    y = None

    def _load(self, data):
        direction = int(data[-2:])
        
        if direction in xrange(64):
            self.power = int(data[-3])
            self.y = int(data[:-6])
            self.x = int(data[-6:-3])
        else:
            self.y = int(data[:-5])
            self.x = int(data[-5:-2])
            
        # item check
        if direction in xrange(32):
            self.weapon = itemList.index(GasGrenade)
            self.direction = direction
        elif direction in xrange(32, 64):
            self.weapon = itemList.index(HEGrenade)
            self.direction = direction - 32
        elif direction in xrange(64, 99):
            self.weapon = itemList.index(C4)
            self.power = direction - 64

    def setItem(self, item):
        self.weapon = itemList.index(item)
        
    def getItem(self):
        return itemList[self.weapon]

    def _generate(self):
        item = self.getItem()
        direction = self.direction
        power = create_number(self.power, 1)
        if item == GasGrenade:
            pass
        elif item == HEGrenade:
            direction += 32
        elif item == C4:
            direction = 64 + self.power
            power = ''
        else:
            raise Exception('unknown weapon')
        return (create_number(self.y, 3) + create_number(self.x, 3) +
                power + create_number(direction, 2))
"""
*** difference! ***
molo: '1150028'
gend: '001150028'
"""
class _GrenadeCommon(PrefixSend):
    x = None
    y = None
    power = None
    direction = None

    def _load(self, data):
        self.y = int(data[:-6])
        self.x = int(data[-6:-3])
        self.power = int(data[-3])
        self.direction = int(data[-2:])
        
    def getWeapon(self):
        return self._weapon
        
    def _generate(self):
        return (str(self.y) + create_number(self.x, 3) + 
                create_number(self.power, 1) + 
                create_number(self.direction, 2))

class ThrowFlashbang(_GrenadeCommon):
    _weapon = Flashbang

class ThrowSmoke(_GrenadeCommon):
    _weapon = SmokeGrenade

class ThrowMolotov(_GrenadeCommon):
    _weapon = MolotovCocktail

    def _load(self, data):
        _GrenadeCommon._load(self, data)
        if data != self._generate():
            print '*** difference! ***'
            print 'molo: %r' % data
            print 'gend: %r' % self._generate()
"""
2030184
"""

class Shoot(PrefixSend):
    showOnly = None
    direction = None
    hitX = None
    hitY = None
    weapon = None

    def _load(self, data):
        if len(data) <= 2:
            self.direction = 0
            self.weapon = int(data)
            self.showOnly = True
        elif len(data) != 8:
            self.direction = int(data[:-2])
            self.weapon = int(data[-2:])
            self.showOnly = True
        else:
            self.hitY = int(data[0:3])
            self.hitX = int(data[3:6])
            self.weapon = int(data[6:8])
            self.showOnly = False
            
    def getWeapon(self):
        return weapons[self.weapon]
        
    def setWeapon(self, weapon):
        self.weapon = find_key(weapons, weapon)
        
    def _generate(self):
        if self.showOnly:
            return str(self.direction) + create_number(self.weapon, 2)
        else:
            return (create_number(self.hitY, 3) + create_number(self.hitX, 3) + 
                    create_number(self.weapon, 2))
"""
203018
"""
class _CommonHit(PrefixSend):
    weapon = None
    targetPlayer = None
    bloodAmount = None
    headshot = None
    damage = None
    
    def _load(self, data):
        if len(data) == 4:
            self.weapon = 0
            self.targetPlayer = int(data[0])
            headshot = int(data[1])
            self.damage = int(data[2:5])
        elif len(data) < 6:
            self.weapon = int(data[0])
            self.targetPlayer = int(data[1:3])
            headshot = int(data[3])
            self.damage = int(data[4:6])
        else:
            self.weapon = int(data[:-5])
            self.targetPlayer = int(data[-5:-3])
            headshot = int(data[-3])
            self.damage = int(data[-2:])
        
        if headshot >= 3:
            self.bloodAmount = headshot-3
            self.headshot = True
        else:
            self.bloodAmount = headshot
            self.headshot = False
        
    def _generate(self):
        if self.headshot:
            bloodAmount = self.bloodAmount + 3
        else:
            if not self.bloodAmount in xrange(3):
                raise Exception('blood amount not within range')
            bloodAmount = self.bloodAmount
        return (create_number(self.weapon, 2) + create_number(self.targetPlayer, 2) +
                create_number(bloodAmount, 1) + create_number(self.damage, 2))

class Hit(_CommonHit):
    def getWeapon(self):
        return weapons[self.weapon]
        
    def setWeapon(self, weapon):
        self.weapon = find_key(weapons, weapon)

class HeavyHit(_CommonHit):
    def getItem(self):
        return items[self.weapon]
        
    def setItem(self, weapon):
        self.weapon = itemToId[weapon]

class Killed(PrefixSend):
    killingPlayer = None
    killedPlayer = None
    dieType = None
    
    def _load(self, data):
        self.dieType = int(data[0])
        self.killedPlayer = int(data[1:3])
        self.killingPlayer = int(data[3:5])
        
    def getDieType(self):
        return dieTypes[self.dieType-1]
        
    def setDieType(self, dieType):
        self.dieType = dieTypes.index(dieType)+1
        
    def _generate(self):
        return (create_number(self.dieType, 1) + create_number(self.killedPlayer, 2) + 
               create_number(self.killingPlayer, 2))