# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

from twisted.internet import reactor

class ConnectionChecker(object):
    handlers = None
    
    attemptsPerMinute = None
    simultaneousConnections = None

    timedPlayers = None
    connectedPlayers = None
    
    def __init__(self, attemptsPerMinute = 30, simultaneousConnections = 5):
        self.handlers = []

        self.attemptsPerMinute = attemptsPerMinute
        self.simultaneousConnections = simultaneousConnections
        
        self.timedPlayers = {}
        self.connectedPlayers = {}

    def playerConnected(self, connection):
        ip = connection.transport.getPeer().host
        connectedPlayers = self.connectedPlayers
        if ip in connectedPlayers:
            connectedPlayers[ip] += 1
        else:
            connectedPlayers[ip] = 1
        if connectedPlayers[ip] > self.simultaneousConnections:
            self._fireHandlers(ip)
            
    def playerDisconnected(self, connection):
        ip = connection.transport.getPeer().host
        connectedPlayers = self.connectedPlayers
        connectedPlayers[ip] -= 1
        if connectedPlayers[ip] == 0:
            del connectedPlayers[ip]
        
    def playerTimedOut(self, connection):
        ip = connection.transport.getPeer().host
        timedPlayers = self.timedPlayers
        if ip in timedPlayers:
            timedPlayers[ip] += 1
        else:
            timedPlayers[ip] = 1
            reactor.callLater(60, self._resetTimeout, ip)
        if timedPlayers[ip] > self.attemptsPerMinute:
            self._fireHandlers(ip)
        
    def _resetTimeout(self, ip):
        del self.timedPlayers[ip]
        
    def _fireHandlers(self, ip):
        for handler in self.handlers:
            handler(ip)
        
    def addHandler(self, handler):
        self.handlers.append(handler)