# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

from twisted.internet import reactor

from pySndo.network.client import ClientGameProtocol, ClientGameFactory, \
    ClientInfoProtocol, ClientInfoFactory

from pySndo.game.weapon import Glock, SIGP, DesertEagle, Benelli, MP5, Galil, \
    SVD, AK47, Colt, G3A3, SR60, AW50, GasGrenade, HEGrenade, C4, Flashbang, \
    MolotovCocktail, SmokeGrenade, Medipack, Other

from pySndo.network.packetloaders.battle import Reload, Shoot, Hit, HeavyHit, \
    Killed, UseItem, ThrowFlashbang, ThrowSmoke, ThrowMolotov, \
    itemToId, weapons, dieWeapons
    
from manhole import createRemoteFactory

class MyGameProtocol(ClientGameProtocol):
    def connectionMade(self):
        try:
            if not self.factory.manholeFactory:
                newManhole = createRemoteFactory(locals(), admin = 'test')
                self.factory.manholeFactory = newManhole
                reactor.listenTCP(2222, newManhole)
        except: 
            pass
        #self.login('Mai Nameh Is Butch')
        self.login('Test321')
        
    def packetReceived(self, packet):
        ClientGameProtocol.packetReceived(self, packet)
        print '<- %s: %s' % (packet.type.__class__.__name__, vars(packet.type))
        
    def packetCorrupted(self, realData, e):
        traceback.print_exc()
        file = open('./bug.txt', 'ab')
        file.writelines(['** corrupted: %r **' % data])
        traceback.print_exc(file = file)
        file.close()
        
    def sendType(self, packetType):
        ClientGameProtocol.sendType(self, packetType)
        print '-> %s: %s' % (packetType.__class__.__name__, vars(packetType))

    def serverClosed(self):
        print 'Server closing...'
        reactor.stop()
        
    def serverFull(self):
        print 'Server full'
        self.transport.loseConnection()
        
    def gotKicked(self):
        print 'Server demanded us to leave the server'
        reactor.stop()
        
    def gotBanned(self):
        print 'Server banned us from the server'
        reactor.stop()
        
    def sessionRestarted(self):
        print 'Session restarted...'
        
    def loggedIn(self):
        print 'Logged in!'
        
    def teamScoresReceived(self, terrorist, police):
        print 'Police: %s' % police, 'Terrorist: %s' % terrorist
    
    def mapChanged(self, newMap):
        print 'Map changed: %s' % newMap.name
        
    def bombedArea(self, x, y):
        print '(got bombed area, %s %s)' % (x, y)
        
    def playerExists(self, player):
        print '%r is playing on the server' % player.name
        
    def playerJoined(self, player):
        print '%r joined the server' % player.name
        
    def playerLeft(self, player):
        print '%r left the server' % player.name
        
    def playerBanned(self, player):
        print '%r banned from the server' % player.name

    def playerKicked(self, player):
        print '%r kicked from the server' % player.name
        
    def playerJoinedTeam(self, player, team):
        print '%r joined team %r' % (player.name, team.__class__.__name__)
        
    def playerChangedTeam(self, player, team):
        print '%r changed team to %r' % (player.name, team.__class__.__name__)
        
    def messageReceived(self, value):
        print '--- %s ---' % value
        
    def statusReceived(self, value):
        print '<Server> %s' % value
        
    def playerMessageReceived(self, player, message):
        print '<%s> %s' % (player.name, message)
        if message.startswith('send '):
            self.sendLine(message[5:])
    
    def teamMessageReceived(self, player, message):
        print '(%s)<%s> %s' % (player.team.__class__.__name__, player.name, message)
        
    def playerMoving(self, player):
        return
        print '%r is moving' % player.name
        
    def playerChangedDirection(self, player):
        return
        print '%r changed direction' % player.name
        
    def playerStopped(self, player):
        return
        print '%r stopped' % player.name
        
    def playerLooked(self, player):
        return
        print '%r looked' % player.name
        
    def playerShot(self, player, weapon, showOnly, hitX, hitY):
        print '%r shot with %s' % (player.name, weapon.name)
        
    def playerHit(self, sourcePlayer, targetPlayer, weapon, 
                  headshot, bloodAmount, damage):
        print '%r hit %r with %s, %s damage (hs: %s, %s)' % (sourcePlayer.name, 
            targetPlayer.name, weapon.name, damage, headshot, bloodAmount)

    def playerKilled(self, killingPlayer, killedPlayer, dieType):
        print '%r killed %r' % (killingPlayer.name, 
            killedPlayer.name)
            
    def gotHit(self, player, weapon, 
                  headshot, bloodAmount, damage):
        print 'Got hit by %r with %s, %s damage (hs: %s, %s)' % (player.name, 
            weapon.name, damage, headshot, bloodAmount)

    def gotKilled(self, player, dieType):
        print 'Got killed by %r' % (player.name)
            
    def playerSpawned(self, player, x, y):
        print '%r spawned at (%s, %s)' % (player.name, x, y)
            
    def playerUsedItem(self, player, item, direction, power):
        print '%r threw %s with %s power' % (player.name, item.name, power)
        
    def crateSpawned(self, crate):
        print '%s crate spawned!' % crate.__class__.__name__
        
    def crateDestroyed(self, crate):
        return
        
    def spawnGranted(self):
        print 'Spawn granted!'
        
    def roundStarted(self):
        print 'Round started'
        
    def roundEnded(wself, winningTeam):
        print 'Round ended, winner: %s' % winningTeam.__class__.__name__
        
class MyGameFactory(ClientGameFactory):
    protocol = MyGameProtocol
    reconnects = 0
    manholeFactory = None
        
    def loadMap(self, mapName):
        import os
        from pySndo.map.loader import Map
        # search path
        path = os.path.join(os.getcwdu(), '..', '..', 'Maps', mapName + '.sdo')
        if not os.path.isfile(path):
            raise Exception('no proper map found')
        newMap = Map(open(path, 'rb').read())
        newMap.name = mapName
        return newMap
        
    def clientConnectionLost(self, connector, reason):
        self.reconnects += 1
        print 'Reconnecting... %s' % self.reconnects
        reactor.callLater(5, connector.connect)

class MyInfoProtocol(ClientInfoProtocol):
    def connectionMade(self):
        print 'connection made'
        ClientInfoProtocol.connectionMade(self)
        
    def packetCorrupted(self, realData, e):
        print 'corrupted'
    def initialReceived(self, initial):
        print 'initial received, joining game'
        self.joinGame()

class MyInfoFactory(ClientInfoFactory):
    protocol = MyInfoProtocol
    gameFactory = MyGameFactory

def main():
    host = '77.252.34.122'
    port = 1204
    mapPort = reactor.connectTCP(host, port, MyInfoFactory())
    print 'Connecting to server...'
    reactor.run()

if __name__ == '__main__':
    main()