# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

from command import Parameter, Command, Commands

from pySndo.map.loader import loadMap

def getPlayer(nameOrId, factory):
    if len(name) <= 2:
        try:
            return [factory.playerFromId(int(nameOrId))]
        except ValueError:
            pass
    return factory.searchPlayers(nameOrId)

def createHelper(commands):
    """
    Returns a function that works as a !help
    handler for 'commands'
    """
    def getHelp(connection, command = None):
        if not command:
            return 'List of commands: ' + ', '.join(['!'+command.name for command in commands.commands 
                if command.rights <= connection.rights])
        else:
            commandObject = commands.findCommand(command)
            if not commandObject:
                return 'Command not found!'
            return commandObject.getDescription()
    return getHelp

def getPing(connection):
    if not connection.latency:
        return 'Ping not retrieved yet, try again'
    return 'Your ping is %s' % int(connection.latency)

def getHp(connection, input = None):
    if not input:
        return 'You have %s HP left' % (connection.player.health or 0)
    try:
        player = getPlayer(input, connection.factory)[0]
    except IndexError:
        return 'Player could not be found'
    if connection.rights < 2:
        if connection.factory.map.data.getGameType() == 'Deathmatch':
            return '(not available in dm mode)'
        elif connection.player.getTeam() != player.player.getTeam():
            return '(not allowed to get from this player)'
    return '%s has %s HP left' % (player.name, (player.player.health or 0))

def sendPm(connection, input, message):
    players = getPlayer(input, connection.factory)
    if not players:
        return 'Player could not be found'
    for player in players:
        player.sendStatus('%s(PM): %s' % (connection.name, message))
    return '(message sent to %s recipients)' % len(players)

def kickPlayer(connection, input, reason):
    try:
        player = getPlayer(input, connection.factory)[0]
    except IndexError:
        return 'Player could not be found'
    player.kick(reason)
    return '%r kicked successfully' % player.name

def banPlayer(connection, input, reason):
    try:
        player = getPlayer(input, connection.factory)[0]
    except IndexError:
        return 'Player could not be found'
    player.kick(reason, True)
    ip = player.transport.getPeer().host
    banned = connection.factory.banned['Bans']
    banned['BannedIPs'].append(ip)
    banned['BannedReasons'][ip] = reason
    connection.factory.saveBans()
    return '%r banned successfully' % player.name

def getIp(connection, input):
    try:
        player = getPlayer(input, connection.factory)[0]
    except IndexError:
        return 'Player could not be found'
    return 'IP of %s: %s' % (player.name, player.transport.getPeer().host)

def loginWithPassword(connection, password):
    for (userPassword, rights) in connection.factory.passwords:
        if userPassword == password:
            connection.rights = rights
            return 'Logged in to the admin interface'
    connection.kick('Invalid login')

def sendNoticable(connection, message):
    newMessage = '%s: %s' % (connection.name, message)
    connection.factory.broadcastMessage(newMessage)

def changeMap(connection, map):
    mapObject = loadMap(map, './../../Maps/')
    if not mapObject:
        return 'Map %r not found' % map
    else:
        connection.factory.changeMap(mapObject)

def startRound(connection, now = False):
    if now:
        delay = 0
    else:
        delay = None
    connection.factory.endRound(delay)

def debugMessage(connection, message):
    print '%r -> %r' % (connection.name, message)
    connection.sendLine(message)

def getCommands():
    commandsObject = Commands()
    commands = [
    
        Command('login', 
            [
                Parameter('password', str, 'Password to attempt to login with')
            ], 'Login to the admin interface', loginWithPassword, 0),
            
        Command('hp', 
            [
                Parameter('player', str, 'Player to get HP of, empty for yourself', True)
            ], 'Get your own or a player on your team\'s current HP', getHp, 0),
            
        Command('map', 
            [
                Parameter('map', str, 'Mapname to switch to')
            ], 'Change to a new map', changeMap, 1),

        Command('kick', 
            [
                Parameter('player', str, 'Player to kick'),
                Parameter('reason', str, 'Reason for the kick', True)
            ], 'Kicks a player', kickPlayer, 1),
            
        Command('ban', 
            [
                Parameter('player', str, 'Player to ban'),
                Parameter('reason', str, 'Reason for the ban', True)
            ], 'Bans a player', banPlayer, 1),

        Command('msg', 
            [
                Parameter('message', str, 'Message to send to clients')
            ], 'Send a noticable message', sendNoticable, 1),
            
        Command('pm', 
            [
                Parameter('Player', str, 'Player to send to'),
                Parameter('message', str, 'Message to send')
            ], 'Send a private message to a specific user', sendPm, 0),
            
        Command('ip', 
            [
                Parameter('Player', str, 'Player to get IP of')
            ], 'Get the IP of a player', getIp, 1),
            
        Command('ping', 
            [], 'Get your ping', getPing, 0),

        Command('startround', 
            [
                Parameter('now', bool, 'Whether the round shall be started *right* now', True)
            ], '(Re)start the round', startRound, 1),
            
        Command('debug', 
            [
                Parameter('message', str, 'Raw string message to send')
            ], 'Used to send raw strings to yourself (for development/test purposes)', debugMessage, 2),
            
        Command('help', 
            [
                Parameter('command', str, 'Command to get help from', True)
            ], 'Get a list of commands available or help for a specific command', createHelper(commandsObject), 0)
    ]
    
    commandsObject.commands = commands
    
    return commandsObject